@@ -1,4 +1,3 @@
-from waffle_utils.file import io
 from waffle_utils.log import datetime_now
 from waffle_utils.utils import type_validator
 
@@ -9,7 +8,7 @@ class Image(BaseField):
     def __init__(
         self,
         # required
-        img_id: int,
+        image_id: int,
         file_name: str,
         width: int,
         height: int,
@@ -17,7 +16,7 @@ class Image(BaseField):
         date_captured: str = None,
     ):
 
-        self.img_id = img_id
+        self.image_id = image_id
         self.file_name = file_name
         self.width = width
         self.height = height
@@ -25,15 +24,17 @@ class Image(BaseField):
 
     # properties
     @property
-    def img_id(self):
-        return self.__img_id
+    def image_id(self):
+        return self.__image_id
 
-    @img_id.setter
+    @image_id.setter
     @type_validator(int)
-    def img_id(self, v):
+    def image_id(self, v):
+        if v is None:
+            raise ValueError("image_id should not be None")
         if v and v < 1:
             raise ValueError("id should be greater than 0.")
-        self.__img_id = v
+        self.__image_id = v
 
     @property
     def file_name(self):
@@ -77,7 +78,7 @@ class Image(BaseField):
     @classmethod
     def new(
         cls,
-        img_id: int,
+        image_id: int,
         file_name: str,
         width: int,
         height: int,
@@ -86,7 +87,7 @@ class Image(BaseField):
         """Image Format
 
         Args:
-            img_id (int): image id. natural number.
+            image_id (int): image id. natural number.
             file_name (str): file name. relative file path.
             width (int): image width.
             height (int): image height.
@@ -95,47 +96,7 @@ class Image(BaseField):
         Returns:
             Image: image class
         """
-        return cls(img_id, file_name, width, height, date_captured)
-
-    @classmethod
-    def from_dict(cls, d: dict) -> "Image":
-        """Image Format from dictionary
-
-        Args:
-            d (dict): Image dictionary
-
-        Returns:
-            Image: image class
-        """
-        img_id = d.get("id", None)
-        file_name = d.get("file_name", None)
-        width = d.get("width", None)
-        height = d.get("height", None)
-        date_captured = d.get("date_captured", datetime_now())
-
-        if img_id is None:
-            raise ValueError("id field missing")
-        if file_name is None:
-            raise ValueError("file_name field missing")
-        if width is None:
-            raise ValueError("width field missing")
-        if height is None:
-            raise ValueError("height field missing")
-
-        return cls(img_id, file_name, width, height, date_captured)
-
-    @classmethod
-    def from_json(cls, f: str) -> "Image":
-        """Image Format from json file
-
-        Args:
-            d (dict): Image json file
-
-        Returns:
-            Image: Image class
-        """
-        d: dict = io.load_json(f)
-        return cls.from_dict(d)
+        return cls(image_id, file_name, width, height, date_captured)
 
     def to_dict(self) -> dict:
         """Get Dictionary of Category
@@ -145,7 +106,7 @@ class Image(BaseField):
         """
 
         cat = {
-            "id": self.img_id,
+            "image_id": self.image_id,
             "file_name": self.file_name,
             "width": self.width,
             "height": self.height,
